from tw.api import Widget, JSLink, CSSLink, JSSource, Link
from tw.extjs import all, all_css, gray_theme

from paraxip.gateway_gui_widgets import debug, base, extjsOverrides

# !+! jpboily : Bug 6623 - ExtJS 2.2 has some issues working with IE 9. To fix
# those issue, we need to force IE to render with a specific version by adding
# the meta tag <meta http-equiv="X-UA-Compatible" content="IE=8" />
# immediately after the <head> tag.
class IECompatibilityMixin:
    def post_init(self, *args, **kw):
        self._resources.add(self)

class IECompatibilityLink(Link, IECompatibilityMixin):
    template = """<meta http-equiv="X-UA-Compatible" content="IE=7" />"""

# end of bug 6623

class TabPstnJS(JSSource):
    _resources = []

    ie_compatibility = IECompatibilityLink()

    alarms_script = \
                    JSLink(link="/static/javascript/widgets_src/status/alarms.js")
    channels_script = \
                    JSLink(link="/static/javascript/widgets_src/status/channels.js")
    controls_script = \
                    JSLink(link="/static/javascript/widgets_src/status/controls.js")
    system_status_script = \
                    JSLink(link="/static/javascript/widgets_src/status/systemStatus.js")
    logging_script = \
                    JSLink(link="/static/javascript/widgets_src/status/logging.js")
    routing_rules_script = \
                    JSLink(link="/static/javascript/widgets_src/routingRules.js")
    filebrowser_script = \
                    JSLink(link="/static/javascript/widgets_src/fileBrowser.js")
    advanced_config_script = \
                    JSLink(link="/static/javascript/widgets_src/advanced.js")
    fxotune_script = \
                    JSLink(link="/static/javascript/widgets_src/fxotune/fxotune.js")
    fxotune_front_form_script = \
                    JSLink(link="/static/javascript/widgets_src/fxotune/fxotune_front_form.js")
    fxotune_front_second_form_script = \
                    JSLink(link="/static/javascript/widgets_src/fxotune/fxotune_front_form_second.js")
    fxotune_select_lines_script = \
                    JSLink(link="/static/javascript/widgets_src/fxotune/fxotune_select_lines.js")
    fxotune_run_calibration = \
                    JSLink(link="/static/javascript/widgets_src/fxotune/fxotune_run_calibration.js")      
    smart_update_script = \
                    JSLink(link="/static/javascript/widgets_src/smartUpdate.js")
    provisioning_script = \
                    JSLink(link="/static/javascript/widgets_src/provisioning.js")
    support_script = \
                    JSLink(link="/static/javascript/widgets_src/support.js")

    javascript=[ie_compatibility,
                all,
                base.check_column_script,
                extjsOverrides.overrides_script,
                base.oam_common_lib_script,
                base.BrowseButton,
                base.FileUploader,
                base.UploadPanel,
                base.FileTreeMenu,
                base.FileTreePanel,
                base.ThemeCombo,
                base.IconCombo,
                base.LangSelectCombo,
                base.css_override, 
                base.filetree_icons, 
                base.filetree_filetree, 
                base.filetree_filetype, 
                base.filetree_famflag, 
                base.filetree_IconCombo, 
                base.filetree_empty, 
                debug.paraxip_console_script,
                alarms_script,
                channels_script,
                controls_script,
                system_status_script,
                logging_script,
                routing_rules_script,
                filebrowser_script,
                advanced_config_script,
                fxotune_front_form_script,
                fxotune_front_second_form_script,
                fxotune_select_lines_script,
                fxotune_run_calibration,
                fxotune_script,
                smart_update_script,
                provisioning_script,
                support_script ]
    
  
    params = source_vars = ["tabs", 
                            "items", 
                            "width", 
                            "height", 
                            "renderTo", 
                            "autoScroll", 
                            "plain",
                            "activeTab", 
                            "frame", 
                            "tabList",
                            "src_file",
                            "offerVirtualBoards",
                            "isGatewayInCLI",
                            "autoDetectHardware",
                            "startGwTimeout"]

    tabs = {'Tab Title 1': ['Tab content 1','Tab id 1','Tab 1'],'Tab Title 2': ['Tab content 2','Tab id 2','Tab 2']}
    items = [{'html': k.replace(" ","").lower(), 'title': k, 'id':v[1], 'tabTip': v[2],'renderTo':k.replace(" ","").lower()} for k, v in tabs.items()]
    tabList = [(k.replace(" ","").lower(),v[0],v[1],v[2]) for k, v in tabs.items()]

    frame = 'false'
    activeTab=0
    width = "300"
    height = "600"
    autoScroll = 'true'
    plain = 'true'

    src=""

    template_engine=" "

    def __init__(self, *args, **kw):
        super(TabPstnJS, self).__init__(*args, **kw)

    def update_params(self, d):
        for param in self.source_vars:
            value = getattr(self, param)
            if isinstance(value, bool):
                d[param] = str(value).lower()
        super(TabPstnJS, self).update_params(d)

    def post_init(self, *args, **kw):
        pass

tabpstn_js = TabPstnJS()

class TabPSTN(Widget):
    tabpstn_js_obj = TabPstnJS
    
    params = js_params =   ["tabs", 
                            "items", 
                            "width", 
                            "height", 
                            "renderTo", 
                            "activeTab", 
                            "frame", 
                            "autoScroll", 
                            "plain", 
                            "tabList",
                            "src_file",
                            "offerVirtualBoards",
                            "isGatewayInCLI",
                            "autoDetectHardware",
                            "startGwTimeout"]

    engine_name = "kid"
    template = """
    <div  id="$renderTo" xmlns="http://www.w3.org/1999/xhtml" xmlns:py="http://purl.org/kid/ns#">
        <div py:for="tab in tabList" id="${tab[0]}">
        </div>
    </div>

    """

    
    def __init__(self, *args, **kw):
        super(TabPSTN, self).__init__(*args, **kw)

    def load_src(self, src_name):
        """
        src_name == dotted.path.to.template (without .ext)
        
        """
        src_file = open(src_name)
        src_obj = src_file.read()
        src_file.close()
        #src_obj =""
        return src_obj

    def __new__(cls, *args, **kw):
        cls = Widget.__new__(cls, *args, **kw)
        d = {}

        #copy in parameters from this class to the javascript class (keeps all of the params in one place)
        for param in cls.js_params:
            value = getattr(cls, param)
            if value is not None:
                d[param] = getattr(cls, param)
                if param == "src_file":
                    cls.tabpstn_js_obj.src = cls.load_src(value)

        tabpstn_js = cls.tabpstn_js_obj(**d)
        cls.javascript = [tabpstn_js,]
        return cls
## !!MIRROR FILE, DO NOT MODIFY!!
