import simplejson

# Helpers to build JSON responses to Ext.form.Action.load and submit

# Base class
class FormActionResponse(object):
  def __init__(self, success):
    self._success = success

# Ext.form.Action.Load response - format:
#{
#    success: true,
#    data: {
#        clientName: "Fred. Olsen Lines",
#        portOfLoading: "FXT",
#    }
#}
class FormActionLoadResponse(FormActionResponse):
  def __init__(self, success, data):
    FormActionResponse.__init__(self, success)
    self._data = data
 
  def __str__(self):
    return simplejson.dumps({'success': self._success, 'data': self._data})
    
class FormActionLoadResponseTotal(FormActionResponse):
  """Returns the result of the request as well as the number of records and
  the data to display."""
  def __init__(self, success, data, count):
    FormActionResponse.__init__(self, success)
    self._data = data
    self._count = count
 
  def __str__(self):
    return simplejson.dumps({'success': self._success,
        'totalCount': self._count, 'data': self._data})
        
class FormActionLoadResponseTotalFilter(FormActionResponse):
  """Returns the result of the request as well as the number of records and
  the data to display."""
  def __init__(self, success, data, count, filter):
    FormActionResponse.__init__(self, success)
    self._data = data
    self._count = count
    self._filter = filter
    
  def __str__(self):
    return simplejson.dumps({'success': self._success,
        'totalCount': self._count, 'data': self._data, 'filter': self._filter})

# Ext.form.Action.Submit response - format:
#{
#    success: false,
#    errors: {
#        clientCode: "Client not found",
#        portOfLoading: "This field must not be null"
#    }
#}
class FormActionSubmitResponse(FormActionResponse):
  def __init__(self, success, errors={}):
    FormActionResponse.__init__(self, success)
    self._errors = errors
 
  def __str__(self):
    response = {'success': self._success}
    if self._errors:
      response['errors'] = self._errors
    return simplejson.dumps(response)

class FormActionUploadResponse(FormActionResponse):
  def __init__(self, success, msgList):
    FormActionResponse.__init__(self, success)
    self._msgList = msgList
 
  def __str__(self):
    response = {'success': self._success}

    response['msgList'] = self._msgList
    return simplejson.dumps(response)
## !!MIRROR FILE, DO NOT MODIFY!!
