import turbogears
from turbogears import controllers, expose
from turbogears import flash, redirect, url
from paraxip.gateway_gui import extjs
from pstnconfig import model
from operator import itemgetter
import simplejson
import os
import netborderPySangoma

import logging
log = logging.getLogger("pstnconfig.controllers")

class BoardsController(controllers.Controller):
#################################################
#                                               #
# Boards methods (Add/delete/get/save)          #
#                                               #
#################################################
  ##########################################################################
  @expose()
  def addBoardFromEditor (self, **data):
    """Adds a board to the actual config from the editor"""
    log.info("Adding board: %s" % data)

    boardData = simplejson.loads(data['newBoardData'])
    boardModel = boardData['boardModel']
    config = model.currentConfig
    assert model.boardsModelInfo.has_key(boardModel)
    boardModelInfo = model.boardsModelInfo[boardModel]
    nextWanpipe = model.getNextWanpipe(config)

    if boardModelInfo['type'] == 'digital':
      
      if boardModelInfo['digitalType'] == 'BRI':      
        digitalBoard = model.addSangomaDigitalBoard(
          config,
          boardModel=boardModel,
          boardID=model.getNextBoardID(config),
          starting_wanpipe=nextWanpipe,
          briModType=boardData['briModType'])
      else:
        digitalBoard = model.addSangomaDigitalBoard(
          config,
          boardModel=boardModel,
          boardID=model.getNextBoardID(config),
          starting_wanpipe=nextWanpipe
      )
      
    elif boardModelInfo['type'] == 'analog':
      # Validate the phoneNumbers before adding the boards and the channel
      # groups
      for phoneNumber in boardData['phoneNumbersArray']:
        if not model.validateChannelGroupID(config, phoneNumber):
          log.error("A channel group with the same ID (%s) already exists." \
                    % phoneNumber)
          return(str(extjs.FormActionSubmitResponse(success=False)))

      board = model.addSangomaAnalogBoard(config,
        boardModel=boardModel,
        boardID=model.getNextBoardID(config),
        wanpipe=nextWanpipe,
        numFxoInterfaces=boardData['numFxoInterfaces'],
        numFxsInterfaces=boardData['numFxsInterfaces'],
        phoneNumbers=boardData['phoneNumbersArray'],
        callerNames=boardData['callerNamesArray'])

    elif boardModelInfo['type'] == 'virtual_digital':
      model.addSangomaVirtualDigitalBoard(config,
        boardID=model.getNextBoardID(config),
        numInterfaces=boardData['interfaceNum'])
    else:
      assert 'Unsupported board model type'


    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # When using the hardware detection mode where boards are automatically
  # found on the system using netborder-list-boards tool, this method will
  # update the boards section of the pstn config using the board(s) and the
  # interface(s) the user chose in the list.
  #
  # The **data parameter contains the following sections:
  #   - boards: JSON string that lists the boards available and if they were
  #             selected or not.
  #   - interfaces: JSON string that lists the interfaces for each boards
  ##########################################################################
  @expose()
  def addRemoveBoards(self, **data):
    """ Update the boards section of the pstn-config.xml file when hardware
        detection mode is used."""
    log.info("addRemoveBoards: updating boards section.")
    log.debug("boards and interfaces data: %s" % data)

    boardsData = simplejson.loads(data['boards'])
    interfacesData = simplejson.loads(data['interfaces'])

    config = model.currentConfig # gateway configuration
    hwCfg = model.hardwareCfg # detected boards configuration

    addedBoard = False
    
    fxoAdded = False
    fxsAdded = False
    for boardData in boardsData:
      # Get the instance of this board in hardware configuration
      boardObj = model.getBoardObject(hwCfg,boardData['id'])

      if boardObj is None:
        log.error("could not find board [%s] in hardware detected config" % boardData['id'])

      # Look in the interfaces data list the information for this board
      ifData = None
      for data in interfacesData:
        if data['boardID'] == boardData['id']:
          ifData = data['interfacesData']

      if ifData is None:
        log.error("addRemoveBoards: could not find interfaces data for board [%s]" % boardData['id'])

      # Try to find a matching board in current configuration
      matchingBoard = self.getMatchingBoard(boardObj, config)

      # The board was not configured in the gateway and the user did not
      # selected it. Do nothing.
      if matchingBoard is None and not boardData['active']:
        log.debug("addRemoveBoards: board [%s] was not selected and is not in the current configuration" % boardData['id'])
        continue

      # The board was not configured in the gateway and the user selected it.
      # Add the board to the boards section of the gateway configuration
      elif matchingBoard is None and boardData['active']:
        log.debug("addRemoveBoards: adding board [%s] to configuration" % boardData['id'])

        if boardData['type'] == "digital":
          model.addSangomaDigitalBoardHwMode(
            config,
            boardModel=boardData['model'],
            boardID=boardData['id'],
            displayName=boardData['displayName'],
            pciBus=boardData['pciBus'],
            pciSlot=boardData['pciSlot'],
            interfacesData=ifData
          )
        elif boardData['type'] == "analog":
          # Also add channel groups for FXS interfaces
          model.addSangomaAnalogBoardHwMode(
            config,
            boardModel=boardData['model'],
            boardID=boardData['id'],
            displayName=boardData['displayName'],
            pciBus=boardData['pciBus'],
            pciSlot=boardData['pciSlot'],
            wanpipe=boardObj.get_wanpipe(),
            interfacesData=ifData
          )
          for interface in ifData:
            if interface["type"]=="FXO":
              fxoAdded = True
            elif interface["type"]=="FXS":
              fxsAdded = True
            
        addedBoard = True

      # The board was configured in the gateway and the user unselected it.
      # Delete the board from the gateway configuration.
      elif matchingBoard and not boardData['active']:
        log.debug("addRemoveBoards: removing board [%s] from configuration" % matchingBoard.get_ID())
        # Board exists in the current psnt config and the user unseleted it:
        # remove the board
        model.deleteBoard(config, matchingBoard.get_ID())

      # The board was configured in the gateway and the user let it selected.
      # Interfaces information might have change. Update it
      elif matchingBoard and boardData['active']:
        log.debug("addRemoveBoards: updating board [%s] info" % matchingBoard.get_ID())
        # update display name
        matchingBoard.set_displayName(boardData['displayName'])
        # update interfaces
        self.updateInterfacesInfo(matchingBoard, ifData)

    #save the config afterward
    model.writeCurrentConfigToFile()

    data = {"addedBoard":addedBoard, "fxoAdded":fxoAdded, "fxsAdded":fxsAdded}
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
    #if addedBoard == True:
    #  return(str('addedBoard'))
    #else:
    #  return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteBoardFromEditor (self, **data):
    """Delete a board to the actual config from the editor"""
    log.info("Deleting board: %s" % data)

    boardsData = simplejson.loads(data['boards'])
    config = model.currentConfig

    for boardInfo in boardsData:
      model.deleteBoard(config, boardInfo['id'])

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a dictionary exposing the values of the board corresponding to
  # the passed id.
  # The fields to be exposed to the user are:
  #   - the id of the board entered in the config file
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (digital in that case)
  #   - The model of the board (A101,A102,A104 or A108)
  #   - The number of interface for this board
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': len(board.get_interface()),
      'primaryClockReference': board.get_primaryClockReference(),
      'interfaceNumberingType': board.get_interfaceNumberingType()
    }
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a dictionary exposing the values of the board corresponding to
  # the passed id.
  # The fields to be exposed to the user are:
  #   - the id of the board entered in the config file
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200)
  #   - The number of interfaces
  #   - The wanpipe number
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': len(board.get_interface()),
      'wanpipe': board.get_wanpipe()
    }
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a dictionary exposing the values of the virtual digital board
  #   corresponding to the passed id.
  # The fields to be exposed to the user are:
  #   - the id of the board entered in the config file
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200)
  #   - The number of interfaces
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmVirtualDigitalBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': len(board.get_interface())
    }
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Saves the modified informations of the board.
  # Editable values are:
  #   - displayName
  #   - wanpipe if it is an analog board
  ##########################################################################
  @expose()
  def saveBoards (self, **data):
    """Saves the modifications to a board entered by the user in the editor"""

    log.info("Saving board informations with data: %s" % data)

    #Decode received data
    boardData = simplejson.loads(data['boardInfo'])

    #Retrieve the corresponding instance for the board id
    board = model.getObjectFromEditorId(boardData['nodeId'])

    #Reassign modified values
    model.saveAttributes(board,boardData)

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a list of all the boards, analog and digital, listed in the
  # current configuration.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200,A101,A102,etc.)
  #   - The number of interfaces
  #   - registered id (hidden field).
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getGridBoardsData(self, id):
    """Returns the boards in the current pstn config."""
    config = model.currentConfig

    data = self.getBoardsData(config)

    for element in data:
      element['nodeId'] = \
        model.getObjectId(model.getBoardObject(config,element['id']))

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the boards, analog and digital, that were detected
  # using the netborder-list-boards tools
  ##########################################################################
  @expose()
  def getAvailableBoards(self, isInWizard = False, js=''):
    """Returns the boards found using the hardware detection mode."""

    log.info("Getting available boards on the system")

    # Run the netborder-list-boards tool again to be sure the current listing
    # is matching the system's state
    model.hardwareCfg = \
      model.parseString(netborderPySangoma.getBoardsListString())
    
    # The config object of the detected boards
    config = model.hardwareCfg
    

    data = self.getBoardsData(config)

    # See if the boards found on the system
    for element in data:
      foundBoard = False

      log.debug("getAvailableBoards: type of element['id']=%s"
                % type(element['id']))
      boardObj = model.getBoardObject(config,element['id'])

      if boardObj is None:
        log.error("getAvailableBoards: could not find the instance for " + \
                  "board [%s]" % element['id'])
        continue

      element['pciBus'] = boardObj.get_pciBus()
      element['pciSlot'] = boardObj.get_pciSlot()
      element['primaryClockReference'] = 0
      element['interfaceNumberingType'] = 'DEFAULT'

      if model.boardsModelInfo[boardObj.get_model()]['type'] == "analog":
        element['wanpipe'] = boardObj.get_wanpipe()

      if not isInWizard:
        matchingBoard = self.getMatchingBoard(boardObj, model.currentConfig)

        if matchingBoard:
          element['displayName'] = matchingBoard.get_displayName()
          element['active'] = True

          if not model.boardsModelInfo[matchingBoard.get_model()]['type'] \
              == "analog":
            element['primaryClockReference'] = \
              matchingBoard.get_primaryClockReference()
            element['interfaceNumberingType'] = \
              matchingBoard.get_interfaceNumberingType()
        else:
          element['active'] = False


    return(str(extjs.FormActionLoadResponse(success=True, data=data)))


  ##########################################################################
  # Returns an array of all the FXO interfaces in the given
  # configuration in format that will be serializable in JSON.
  #
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  ##########################################################################
  @expose()
  def getFxoInterfacesData(self, config):
    data = []

    # Append digital boards data
    for board in ( config.get_boards().get_sangomaAnalogBoard()):

      boardType = model.boardsModelInfo[boardModel]['type']
      if boardType == "analog":
        for interface in board.get_interface():
          if interface.getType() == "FXO":
            data.append({
                'id': board.get_ID(),
                'displayName': board.get_displayName() 
                });
      
    
    data = sorted(data, key=itemgetter('id'))
    return data


  ##########################################################################
  # Returns an array of all the boards, analog and digital in the given
  # configuration in format that will be serializable in JSON.
  ##########################################################################
  def getBoardsData(self, config):
    data = []

    # Append digital boards data
    for board in (config.get_boards().get_sangomaDigitalBoard() + \
        config.get_boards().get_sangomaAnalogBoard() + \
        config.get_boards().get_sangomaVirtualDigitalBoard()):

      boardModel = board.get_model() #get the board model
      boardType = model.boardsModelInfo[boardModel]['type']
      numIf = len(board.get_interface())
      data.append({
        'id': board.get_ID(),
        'displayName': board.get_displayName(),
        'type': boardType,
        'model': boardModel,
        'numInterfaces': numIf
      })
    data = sorted(data, key=itemgetter('id'))
    return data


  ##########################################################################
  # Returns the instance of the board in config that matches the given board
  # or None if not found. The match is done by matching the board model, the
  # pciBus and the pciSlot.
  ##########################################################################
  def getMatchingBoard(self, board, config):

    if config is None or board is None:
      return None

    model = board.get_model()
    bus   = board.get_pciBus()
    slot  = board.get_pciSlot()

    for b in config.get_boards().get_sangomaDigitalBoard() + \
        config.get_boards().get_sangomaAnalogBoard():

      if  model == b.get_model() and \
          bus == b.get_pciBus() and \
          slot == b.get_pciSlot():
        return b

    return None

#################################################
#                                               #
# Interfaces methods (Add/delete/get/save)      #
#                                               #
#################################################

  ##########################################################################
  # Updates the information of interfaces for a board. Could consist of
  # simply changing the display name or adding/removing interfaces from a
  # board config
  ##########################################################################
  def updateInterfacesInfo(self, board, interfacesData):
    """Updates the interfaces information for a board."""
    log.info("Updating interfaces information")
    gwInterfaces = board.get_interface()
    boardType = model.boardsModelInfo[board.get_model()]['type']

    for data in interfacesData:
      foundInterface = False

      for interface in gwInterfaces:

        if interface.get_ID() == data['id']:
          foundInterface = True

          # Interface was configured but not wanted anymore. Delete it from
          # the gateway configuration
          if not data['active']:
            model.deleteInterface(
              model.currentConfig,
              board.get_ID(),
              interface.get_ID()
            )
          else:
            interface.set_displayName(data['displayName'])
            if boardType == "analog":
              if data['type'] == "FXS":
                self.changeChannelGroupID(data['phoneNumber'], #newID
                                          interface.get_phoneNumber() ) #oldID
                interface.set_callerName(data['callerName'])
              interface.set_phoneNumber(data['phoneNumber'])

          break # end of gwInterfaces loop

      # Interface was not configured in the gateway and the user selected it.
      # Add it to the gateway configuration.
      if not foundInterface and data['active']:
        if boardType == "digital":
          model.addInterface(
            gwInterfaces,
            ID=data['id'],
            displayName=data['displayName'],
            wanpipe=data['wanpipe']
          )
        elif boardType == "analog":
          model.addInterface(
            gwInterfaces,
            ID=data['id'],
            displayName=data['displayName'],
            type=data['type'],
            wanpipeInterface=data['wanpipeInterface'],
            phoneNumber=data['phoneNumber'],
            callerName=data['callerName']
          )
          if data['type'] == "FXS":
            model.addChannelGroup(
              model.currentConfig,
              ID=data['phoneNumber'],
              direction="BIDIR",
              outboundHuntingScheme="REVERSE_LINEAR"
            )


  ##########################################################################
  #
  ##########################################################################
  @expose()
  def addInterfaceFromEditor (self, **data):
    """Adds an interface to a board from the editor"""

    log.info("Adding interface with following data: %s" % data)

    boardID = simplejson.loads(data['boardID'])
    log.debug("Board ID: %s" % boardID)

    interfaceData = simplejson.loads(data['interfaceData'])
    log.debug("Interface data: %s" % interfaceData)

    cfg = model.currentConfig

    board = model.getBoardObject(cfg, boardID)

    boardType = model.boardsModelInfo[board.get_model()]['type']

    interfaceList = board.get_interface()

    phoneNumber = None
    callerName = None

    if model.hardwareDetectionEnabled:
      log.info("not implemented yet.")
    else:
      if boardType == "digital":
        model.addInterface(
          interfaceList,
          ID=model.getNextInterfaceID(cfg,boardID),
          displayName=interfaceData['displayName'],
          wanpipe=model.getNextWanpipe(cfg)
        )
      elif boardType == "analog":
        phoneNumber = interfaceData['phoneNumber']
        if interfaceData['type'] == "FXS":
          callerName = interfaceData['callerName']
          if not model.validateChannelGroupID(cfg, phoneNumber):
            log.error("A channel group with the same ID (%s) already exists." \
                      % phoneNumber)
            return(str(extjs.FormActionSubmitResponse(success=False)))
          model.addChannelGroup(
            cfg,
            ID=phoneNumber,
            direction="BIDIR",
            outboundHuntingScheme="REVERSE_LINEAR"
          )

        model.addInterface(
          interfaceList,
          ID=model.getNextInterfaceID(cfg,boardID),
          displayName=interfaceData['displayName'],
          wanpipeInterface=model.getNextWanpipeInterface(cfg, boardID),
          type=interfaceData['type'],
          phoneNumber=phoneNumber,
          callerName=callerName
        )
      elif boardType == "virtual_digital":
        model.addInterface(
          interfaceList,
          ID=model.getNextInterfaceID(cfg,boardID),
          displayName=interfaceData['displayName'],
          peerVirtualInterfaceID=model.getNextPeerVirtualInterfaceID(cfg, boardID)
        )

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  #
  ##########################################################################
  @expose()
  def deleteInterfaceFromEditor (self, **data):
    """Delete an interface from a board from the editor"""

    log.info("Deleting interfaces: %s" % data)

    boardID = simplejson.loads(data['boardID'])
    interfaces = simplejson.loads(data['interfaces'])

    config = model.currentConfig

    for interface in interfaces:
      model.deleteInterface(config, boardID, interface['id'])

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - the id of the interface entered in the config file
  #   - The display name of the interface
  #   - The wanpipe number of this interface
  #   - registered id (hidden field).
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    briModType = ""
    if interface.get_briModType() == "TERMINAL" or \
       interface.get_briModType() == "NETWORK":
      briModType = interface.get_briModType()
    else:
      briModType = "N/A"
      
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'wanpipe': interface.get_wanpipe(),
      'nodeId': id,
      'briModType':briModType
    }
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - the id of the interface entered in the config file
  #   - The display name of the interface
  #   - The wanpipeInterface number of this interface
  #   - registered id (hidden field).
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'wanpipeInterface': interface.get_wanpipeInterface(),
      'nodeId': id,
      'interfaceType': interface.get_type(),
      'phoneNumber': interface.get_phoneNumber(),
      'callerName': interface.get_callerName()
    }

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the virutal digital interface associated to the
  #   given id.
  # The fields to be exposed to the user are:
  #   - the id of the interface entered in the config file
  #   - The display name of the interface
  #   - The peerVirtualInterfaceID of this interface
  #   - registered id (hidden field).
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmVirtualDigitalInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'peerVirtualInterfaceID': interface.get_peerVirtualInterfaceID(),
      'nodeId': id
    }
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Saves the modified informations of the interface.
  # Editable values are:
  #   - displayName
  #   - peerVirtualInterfaceID if it is a virtual board
  #   - wanpipeInterface if it is an analog board
  ##########################################################################
  @expose()
  def saveInterfaces (self, **data):
    """Saves the modifications to an interface entered by the user in the editor"""
    log.info("Saving interface informations with data: %s" % data)

    #Decode received data
    interfaceData = simplejson.loads(data['interfaceInfo'])
    #Retrieve the corresponding instance for the board id
    interface = model.getObjectFromEditorId(interfaceData['nodeId'])
    #Reassign modified values
    model.saveAttributes(interface,interfaceData)

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Returns a list of interface for the digital board's given id.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The wanpipe number of this interface
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getGridDigitalInterfacesData(self, id):
    # Get the instance of the board
    board = model.getObjectFromEditorId(id)

    data = self.getDigitalInterfacesData(board)

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of interface for the analog board's given id.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The wanpipeInterface number of this interface
  ##########################################################################
  @expose() # Ext.form.Action.load - expects JSON
  def getGridAnalogInterfacesData(self, id):
    # Get the instance of the board
    board = model.getObjectFromEditorId(id)

    data = self.getAnalogInterfacesData(board)

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  
    
    
    
  @expose()
  def getAvailableInterfaces(self, boardID, isInWizard = False):
    """Returns the interfaces available for this board"""

    id = int(boardID)

    config = model.hardwareCfg

    log.debug("getAvailableInterfaces: type of id=%s" % type(id))
    board = model.getBoardObject(config,id)

    if board is None:
      log.error("getAvailableInterfaces: board [%s] was not found in the " + \
                "detected boards" % id)
      return(str(extjs.FormActionLoadResponse(success=False, data=[])))

    boardType = model.boardsModelInfo[board.get_model()]['type']

    if boardType == "digital":
      data = self.getDigitalInterfacesData(board)
    elif boardType == "analog":
      data = self.getAnalogInterfacesData(board)

    # Try to find the board in the gateway configuration
    cfgBoard = self.getMatchingBoard(board, model.currentConfig)

    for element in data:
      # For each element in the array corresponding to an interface
      foundInterface = False

      if cfgBoard and not isInWizard:
        # If the board was already configured in the gateway
        for interface in cfgBoard.get_interface():
          # For each configured interface for that board
          if element['id'] == interface.get_ID():
            # If the interface in the gateway configuration matches the
            # hardware interface, update the information of the interface that
            # that will be returned to the UI with the information of the
            # configured interface
            element['displayName'] = interface.get_displayName()
            element['active'] = True
            if boardType == "analog":
              element['phoneNumber'] = interface.get_phoneNumber()
              element['callerName'] = interface.get_callerName()
            foundInterface = True

      if not foundInterface:
        element['active'] = False

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  def getDigitalInterfacesData(self, board):
    data = []
   
    for i in board.get_interface():
      
      boardModel = board.get_model()
      digitalType = model.boardsModelInfo[boardModel]['digitalType']
      
      interface = {
        'id': i.get_ID(),
        'displayName': i.get_displayName(),
        'wanpipe': i.get_wanpipe(),
        'nodeId': model.getObjectId(i),
      }
      if(digitalType == 'BRI'):
        interface['briModType'] = i.get_briModType()
        
      data.append(interface)
      
    data = sorted(data, key=itemgetter('id'))
    return data

  def getAnalogInterfacesData(self, board):
    data = []

    for i in board.get_interface():
      ifType = i.get_type()
      interfaceData = {
        'id': i.get_ID(),
        'displayName': i.get_displayName(),
        'wanpipeInterface': i.get_wanpipeInterface(),
        'type': ifType,
        'nodeId': model.getObjectId(i),
        'phoneNumber': i.get_phoneNumber(),
        'callerName': i.get_callerName()
      }
      data.append(interfaceData)

    data = sorted(data, key=itemgetter('id'))
    return data

  ##########################################################################
  # Change the ID of the channel group that has for ID the value of oldID
  # for the value of newID.
  # @param newID: the new value for the channel group's ID
  # @param oldID: the ID of the channel group to modify
  #
  # @return success=False if the newID is already taken or if we are unable
  #         to change the ID of the channel group; success=True if
  #         everything worked.
  ##########################################################################
  @expose()
  def changeChannelGroupID(self, newID, oldID):
    config = model.currentConfig
    log.info("Changing ID of the channel group from [%s] to [%s]." \
              % (oldID,newID))
    if not model.validateChannelGroupID(config, newID):
      log.error("The ID [%s] is already taken by another channel group" % newID)
      return(str(extjs.FormActionSubmitResponse(success=False)))

    channelGroup = model.getChannelGroupObject(config, oldID)

    if channelGroup is None:
      log.error("Can not locate channel group with ID [%s]" % oldID)
      return(str(extjs.FormActionSubmitResponse(success=False)))

    channelGroup.set_ID(newID)

    newChannelGroup = model.getChannelGroupObject(config, newID)

    if newChannelGroup is None:
      log.error("Could not locate channel group with new ID [%s]. Reverting to previous ID" % newID)
      channelGroup.set_ID(oldID)
      return

    # Change the ID in the FXO/FXS config as well
    analog = config.get_callControl().get_analog()
    for config in (analog.get_analogFxsConfig()+analog.get_analogFxoConfig()):
      for interfaceRef in model.getInterfaceRefsForConfig(config):
        if interfaceRef.get_channelGroupID() == oldID:
          interfaceRef.set_channelGroupID(newID)

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))
