import logging

import os
import sys
import stat
import simplejson
import tarfile
from netborderPyInfra import PyGlobalConfig as globalConfig
from paraxip.gateway_gui import extjs
from turbogears import controllers, expose, error_handler, validate
from turbogears import flash, redirect, url
from cherrypy.lib.cptools import serve_file

log = logging.getLogger("pstnconfig")

class SupportController(controllers.Controller):

    def __init__(self):
        self.installDir = globalConfig.getParameter("netborder.Installation.Directory")
        log.debug("SupportController::installDir var is: %s" % self.installDir)
        self.archiveFilePath = os.path.join(self.installDir, "nbe_support_package.tgz" )
        log.debug("SupportController::archiveFilePath var is: %s" % self.archiveFilePath)

    # Add file from a gateway sub folder files to the specified tar
    # Exceptions from tarfile is propagated to caller
    def addFolderFiles(self, tar, folder, folderArchName):
        try:
            folderPath = os.path.join(self.installDir, folder)
            fileList = os.listdir(folderPath)
            for name in fileList:
                filePath = os.path.join(folderPath, name)
                if os.path.isfile(filePath):
                    log.info( "SupportController: add file %s" % filePath )
                    tar.add(filePath, folderArchName + name)
        except os.error:
           return
        return

    # Add folder content recusively
    def addFolder(self, tar, folder, folderArchName):
        try:
            folderPath = os.path.join(self.installDir, folder)
            log.info( "SupportController: add folder %s" % folderPath )
            tar.add(folderPath, folderArchName)
        except os.error:
           return
        return
    
    @expose()
    def generateSupportArchive(self, **js):
        log.debug("SupportController::generateSupportArchive")
        
        log.error("upportController::generateSupportArchive: js=%s" % js)
        js = simplejson.loads(js['content'])
        archiveContent = js['archiveContent']
        log.error("Archive will contents: %s" % archiveContent)
        try:
            log.info( "SupportController::generateSupportArchive: Openning tar file %s" % \
                       self.archiveFilePath )
            tar = tarfile.open(self.archiveFilePath, "w:gz")

            # Add config files
            if archiveContent['configFiles']:
                self.addFolderFiles(tar, 'config', 'config/')
            else:
                log.info( "SupportController::generateSupportArchive: not adding config files" ) 

            # Add core-dump files
            if archiveContent['coreFiles']:
                if sys.platform == 'linux':
                    # temporary for linux, the core dumps are
                    # generated under bin folder
                    self.addFolderFiles(tar, 'bin', 'core-dumps/')
                else:
                    self.addFolderFiles(tar, 'logs/core-dumps', 'core-dumps/')
            else:
                log.info( "SupportController::generateSupportArchive: not adding core-dump files" ) 

            # Add call logs files
            if archiveContent['callLogFiles']:
                self.addFolder(tar, 'logs/call-logs', 'call-logs/')
            else:
                log.info( "SupportController::generateSupportArchive: not adding callLog files" ) 

            # Add audio recording logs files
            if archiveContent['audioFiles']:
                self.addFolder(tar, 'logs/recordings', 'recordings/')
            else:
                log.info( "SupportController::generateSupportArchive: not adding recording files" ) 

            # Add log files
            if archiveContent['logFiles']:
                self.addFolderFiles(tar, 'logs', 'logs/')
            else:
                log.info( "SupportController::generateSupportArchive: not adding log files" ) 

            tar.close()

        except tarfile.TarError, (errno, strerror):
            errorMsg = "Failed to create support archive - %s : tar error(%s) - %s" % \
                       (self.archiveFilePath, errno, strerror)
            log.error( errorMsg )
            return(str(extjs.FormActionLoadResponse(success=False,
                                                    data=[{"errorMsg":errorMsg }])))
            
        return (str(extjs.FormActionSubmitResponse(success=True)))

    @expose()
    def downloadSupportArchive(self):
        return serve_file(self.archiveFilePath,
                          "application/force-download", "attachment")
