//-----------------------------------------------
// Add/Delete Channel Group methods
//-----------------------------------------------
Ext.namespace("Paraxip.config.channelGroup");

function addChanGroupHandler(){
  addChannelGroupWindow.show();
};

function addChanGroup(){
  var form = addChannelGroupForm.getForm();
  var el  = form.getEl();

  el.dom.action = Paraxip.baseUrl + "call/addChannelGroup";
  el.dom.method = 'POST';
  var channelGroup = Ext.util.JSON.encode(form.getValues());
  form.submit({
    params: {'channelGroup':channelGroup},
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      addChannelGroupWindow.hide();
      //gridChannelGroup.store.reload();
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });
};

function deleteChanGroupHandler(){
  var selectedRecords = gridChannelGroup.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    if ( gridChannelGroup.store.getCount() - selectedRecords.length < 1 ){
      Ext.Msg.alert("Can not delete last channelGroup!",
        "At least one channelGroup is necessary for the system to work properly. " +
        "Please edit current config or create a new one before deleting " +
        "this one");
    } else {
      var channelGroups = new Array();
      for (var i = 0; i < selectedRecords.length; i++) {
        channelGroups.push(selectedRecords[i].data);
      }
      channelGroups = Ext.util.JSON.encode(channelGroups);
      Ext.Ajax.request({
        url: Paraxip.baseUrl + "call/deleteChannelGroup",
        method: 'POST',
        disableCaching: false,
        params: {"channelGroups":channelGroups},
        success: function ()
        {
          Paraxip.isConfigOutOfSynch(true);
          for (var index = 0; index < selectedRecords.length; index++) {
            gridChannelGroup.store.remove(selectedRecords[index]);
          }
          reloadTree(true);
        },
        failure: Paraxip.failureMessage
      });
    }
  }
};
var newChanGrpID = new Ext.form.TextField({
  id          : "Paraxip.config.channelGroup.chanGrpID",
  xtype       : 'textfield',
  fieldLabel  : "ID",
  name        : "id",
  width       : 200 ,
  allowBlank  : false
});

var newChanGrpDirection = new Ext.form.ComboBox({
  id              : "Paraxip.config.channelGroup.chanGrpDirection",
  xtype           : 'combobox',
  name            : 'direction',
  fieldLabel      : "Direction",
  store           : ['IN','OUT', 'BIDIR'],
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  selectOnFocus   : true,
  width           : 145,
  minListWidth    : 145,
  value           : 'BIDIR'
});

newChanGrpDirection.on('select',function(e){
  if( e.getValue() == 'IN' ) {
    newChangGrpOutbound.disable();
    newChangGrpOutbound.getEl().up('.x-form-item').setDisplayed(false);
  } else {
    newChangGrpOutbound.enable();
    newChangGrpOutbound.getEl().up('.x-form-item').setDisplayed(true);
  }
});

var newChangGrpOutbound = new Ext.form.ComboBox({
  id              : "Paraxip.config.channelGroup.outbound",
  xtype           : 'combobox',
  name            : 'outboundHuntingScheme',
  fieldLabel      : "Outbound Hunting Scheme",
  helpText        : "outboundHuntingSchemeoutboundHuntingScheme",
  store           : ['LINEAR','REVERSE_LINEAR'],
  allowBlank      : false,
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  selectOnFocus   : true,
  width           : 145,
  minListWidth    : 145,
  value           : 'REVERSE_LINEAR'
});

var addChannelGroupForm = new Ext.FormPanel({
  id          : "addChannelGroupForm",
  region      : 'center',
  labelWidth  : 175,
  labelPad    : 10,
  bodyStyle   : 'padding:10px;',
  frame       : false,
  header      : false,
  footer      : false,
  defaults    : {
    "anchor": "75%"
  },
  buttonAlign : "right",
  buttons     : [{
    id: "Paraxip.config.channelGroup.addChanGrpButton",
    text: "Add",
    listeners: {'click':addChanGroup}
  },{
    id: "Paraxip.config.channelGroup.cancelAddChanGrpButton",
    text: "Cancel",
    listeners: {'click':function(){addChannelGroupWindow.hide();}}
  }]
});

addChannelGroupForm.add(newChanGrpID);
addChannelGroupForm.add(newChanGrpDirection);
addChannelGroupForm.add(newChangGrpOutbound);

var addChannelGroupWindow = new Ext.Window({
  id          : "addChannelGroupWindow",
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true
});

addChannelGroupWindow.add(addChannelGroupForm);

var gridChannelGroup = new Ext.grid.GridPanel({
  title:'Channel Group Grid',
  id:'gridChannelGroup',
  autoScroll:true,
  layout      : 'absolute',
  store: new Ext.data.JsonStore({
    id:'storeChannelGroup',
    url: Paraxip.baseUrl + "call/getGridChannelGroupData",
    root: 'data',
    fields: [
      {name: 'id'},
      {name: 'direction'},
      {name: 'outboundHuntingScheme'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns:[
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding:5px;'},
    {header: "Direction", width: 75, sortable: true, dataIndex: 'direction', css:'padding:5px;'},
    {header: "Outbound Hunting Scheme", width: 100, sortable: true, dataIndex: 'outboundHuntingScheme', css:'padding:5px;'}
  ],
  viewConfig:{forceFit:true}
});

gridChannelGroup.addButton("Add ChannelGroup",addChanGroupHandler,gridChannelGroup);
gridChannelGroup.addButton("Remove ChannelGroup(s)",deleteChanGroupHandler,gridChannelGroup);

gridChannelGroup.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

//------------------------------------------------
// Channel Group form
// -----------------------------------------------
function saveChannelGroup(e){
  var owner = e.ownerCt;
  var form = owner.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "call/saveChannelGroup";
  el.dom.method = 'POST';

  var channelGroup = Ext.util.JSON.encode(form.getValues());

  form.submit({
    params:{
      'channelGroup': channelGroup
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

function showOutboundScheme(){
  if( direction_select.getValue() == "IN" ) {
    outboundHuntingScheme_select.disable();
    outboundHuntingScheme_select.getEl().up('.x-form-item').setDisplayed(false);
  } else {
    outboundHuntingScheme_select.enable();
    outboundHuntingScheme_select.getEl().up('.x-form-item').setDisplayed(true);
  }
};

var frmChannelGroup = new Ext.FormPanel({
  title         : "Digital Physical Configuration",
  frame         : true,
  header        : false,
  id            : "frmChannelGroup",
  autoScroll    : true,
  layout        : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },
  defaults    : {
    layout: 'form',
    style: 'text-align:left;',
    labelWidth: 150,
    bodyStyle: 'padding:5px;',
    width: 500,
    autoHeight: true,
    fill: true
  },
  buttonAlign     : "right",
  buttons         : [{
    id: "Paraxip.config.channelGroup.chanGrpSaveButton",
    text: 'Save',
    listeners: {'click':saveChannelGroup}
  },{
    id: "Paraxip.config.channelGroup.chanGrpCancelButton",
    text: 'Cancel',
    listeners: {'click':callControlCancel}
  }]
});

var id_textChanGroup = createIdText({
  id          : 'id_textChanGroup',
  helpText    : "channelGroupIDID"
});

var direction_select = new Ext.form.ComboBox({
  id              : 'direction_selection',
  name            : 'direction',
  fieldLabel      : "Direction",
  store           : ['IN','OUT', 'BIDIR'],
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  selectOnFocus   : true,
  width           : 145,
  minListWidth    : 145
});

direction_select.on('select',function(e){
  if( e.getValue() == 'IN' ) {
    outboundHuntingScheme_select.disable();
    outboundHuntingScheme_select.getEl().up('.x-form-item').setDisplayed(false);
  } else {
    outboundHuntingScheme_select.enable();
    outboundHuntingScheme_select.getEl().up('.x-form-item').setDisplayed(true);
    outboundHuntingScheme_select.setValue("REVERSE_LINEAR");
  }
});

var outboundHuntingScheme_select = new Ext.form.ComboBox({
  id              : 'outboundHuntingScheme_selection',
  name            : 'outboundHuntingScheme',
  fieldLabel      : "Outbound Hunting Scheme",
  helpText        : "outboundHuntingSchemeoutboundHuntingScheme",
  store           : ['LINEAR','REVERSE_LINEAR'],
  allowBlank      : false,
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  selectOnFocus   : true,
  width           : 145,
  minListWidth    : 145,
  value           : 'LINEAR'
});

var ChannelGroupPanel = new Ext.Panel({
  id    : 'ChannelGroupPanel',
  name  : 'channelGroup',
  title : 'Channel Group',
  items : [{
    xtype: 'textfield',
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  }]
});

ChannelGroupPanel.add(id_textChanGroup);
ChannelGroupPanel.add(direction_select);
ChannelGroupPanel.add(outboundHuntingScheme_select);
frmChannelGroup.add(ChannelGroupPanel);

