//-----------------------------------------------
// Add/Delete ISDN configuration methods
//-----------------------------------------------
Ext.namespace("Paraxip.config.callControl");

var addIsdnConfHandler = function(){
  addIsdnConfWindow.show();
};

function addIsdnConfig(){
  var form = addIsdnConfigForm.getForm();
  var el  = form.getEl();

  el.dom.action = Paraxip.baseUrl + "call/addIsdnConfig";
  el.dom.method = 'POST';
  var isdnConfig = Ext.util.JSON.encode(form.getValues());
  form.submit({
    params: {'isdnConfig':isdnConfig},
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      addIsdnConfWindow.hide();
      gridISDNConfiguration.store.reload();
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });
};



var deleteIsdnConfHandler = function(){
  var selectedRecords = gridISDNConfiguration.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    var isdnConfigs = new Array();
    for (var i = 0; i < selectedRecords.length; i++) {
      isdnConfigs.push(selectedRecords[i].data);
    }
    isdnConfigs = Ext.util.JSON.encode(isdnConfigs);
    Ext.Ajax.request({
      url: Paraxip.baseUrl + "call/deleteIsdnConfig",
      method: 'POST',
      disableCaching: false,
      params: {"isdnConfigs":isdnConfigs},
      success: function(result)
      {
        var deleteResponse = Ext.decode(result.responseText);
        if(deleteResponse.success){
          Paraxip.isConfigOutOfSynch(true);
          for (var index = 0; index < selectedRecords.length; index++) {
            gridISDNConfiguration.store.remove(selectedRecords[index]);
          }
          reloadTree(true);
        }
        else{
          Ext.Msg.alert("Can not delete last configuration!",
          "At least one configuration is necessary for the system to work properly. " +
          "Please edit current configuration or create a new one before deleting " +
          "this one");
        }
      },
      failure: Paraxip.failureMessage
    });
  }
};

var isdnConfigChoice = new Ext.form.ComboBox({
  id              : 'isdnConfigChoice',
  fieldLabel      : "ISDN Config Type",
  store           : ['FAS','NFAS', 'BRI'],
  forceSelection  : true,
  triggerAction   : 'all',
  mode            : 'local',
  typeAhead       : true,
  value           : 'FAS',
  selectOnFocus   : true,
  width           : 100
});

var addIsdnConfigForm = new Ext.FormPanel({
  id          : "addIsdnConfigForm",
  region      : 'center',
  labelWidth  : 175,
  labelPad    : 10,
  bodyStyle   : 'padding:10px;',
  frame       : false,
  header      : false,
  footer      : false,
  buttonAlign : "right",
  buttons     : [{
    id: "Paraxip.config.callControl.AddIsdnConfigButton",
    text: "Add",
    listeners: {'click':addIsdnConfig}
  },{
    id: "Paraxip.config.callControl.cancelAddIsdnConfigButton",
    text: "Cancel",
    listeners: {'click':function(){addIsdnConfWindow.hide();}}
  }]
});

addIsdnConfigForm.add(isdnConfigChoice);

var addIsdnConfWindow = new Ext.Window({
  id          : "addIsdnWindow",
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true
});

addIsdnConfWindow.add(addIsdnConfigForm);

var gridISDNConfiguration = new Ext.grid.GridPanel({
  title       : 'ISDN Configuration Grid',
  id          : 'gridISDNConfiguration',
  autoScroll  : true,
  layout      : 'absolute',
  store       : new Ext.data.JsonStore({
    id:'storeISDNConfiguration',
    url: Paraxip.baseUrl + "call/getGridISDNConfigurationData",
    root: 'data',
    fields: [
      {name: 'id'},
      {name: 'displayName'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     : [
    {header: "ID", width: 35, sortable: true, dataIndex: 'id', css:'padding:5px;'},
    {header: "Name", width: 200, sortable: true, dataIndex: 'displayName', css:'padding:5px;'}
  ],
  viewConfig  : {forceFit:true}
});

gridISDNConfiguration.addButton("Add ISDN Config",addIsdnConfHandler,
                                gridISDNConfiguration);
gridISDNConfiguration.addButton("Remove ISDN Config(s)",deleteIsdnConfHandler,
                                gridISDNConfiguration);

gridISDNConfiguration.on('rowdblclick', function(grid, rowIndex){
  var record = grid.store.getAt(rowIndex);
  goToNode(record.get('nodeId'));
});

function isWaitFacilityDelayEnable(){
  var waitFacility;
  if( this.id == "callerNameLocationFas_select" ) {
    waitFacility = waitFacilityDelayMsFas_numfield;
  } else if( this.id == "callerNameLocationNfas_select" ) {
    waitFacility = waitFacilityDelayMsNfas_numfield;
  } else if( this.id == "callerNameLocationBri_select" ) {
    waitFacility = waitFacilityDelayMsBri_numfield;
  }

  if( this.getValue() != "DISABLED" ) {
    waitFacility.enable();
  } else {
    waitFacility.disable();
    waitFacility.clearInvalid();
  }
}

function updateProgressTonesIndicator_combobox(){
  var progressTonesIndicator;
  var switchVariant
  
  if( this.id == "switchVariantFas_select" ) {
    progressTonesIndicator = inbandProgTonesIndFas_select;
    switchVariant = switchVariantFas_select;
  } else if( this.id == "switchVariantNfas_select" ) {
    progressTonesIndicator = inbandProgTonesIndNfas_select;
    switchVariant = switchVariantNfas_select;
  } else if( this.id == "switchVariantBri_select" ) {
    progressTonesIndicator = inbandProgTonesIndBri_select;
    switchVariant = switchVariantBri_select;
  }
   
  switchVariantValue = switchVariant.getValue();

  var data = [];
  //if(switchVariantValue == '4ESS' || switchVariantValue == '5ESS'){
  if(switchVariantValue == 'DMS100'){
    data = ['ALERTING','PROGRESS'];
    
    if(progressTonesIndicator.getValue() == "BOTH")
      progressTonesIndicator.setValue("PROGRESS");
  }
  else
    data = ['ALERTING','PROGRESS','BOTH'];
  
  progressTonesIndicator.store.loadData(data);
}

function isOverlapDigitsReceptionEnabled(){

  if(this.id == "overlapDigitsReceptionBri_select"){

    if(this.getValue() == "ENABLED"){

      //Enable the disabled fields
      minNumberOfDigitsBri_numfield.enable();
      maxNumberOfDigitsBri_numfield.enable();
      endCollectionDigitsBri_text.enable();

      //specify default values if not defined
      if(minNumberOfDigitsBri_numfield.getValue() == '')
        minNumberOfDigitsBri_numfield.setValue(0);
      if(maxNumberOfDigitsBri_numfield.getValue() == '')
        maxNumberOfDigitsBri_numfield.setValue(0);
      if( endCollectionDigitsBri_text.getValue() == '')
        endCollectionDigitsBri_text.setValue("([0-9a-dA-D]+)?[\\#\\*]$");
    }else{
      //disable the enabled fields
      minNumberOfDigitsBri_numfield.disable();
      minNumberOfDigitsBri_numfield.clearInvalid();
      maxNumberOfDigitsBri_numfield.disable();
      maxNumberOfDigitsBri_numfield.clearInvalid();
      endCollectionDigitsBri_text.disable();
      endCollectionDigitsBri_text.clearInvalid();
    }
    
  }else if(this.id == "overlapDigitsReceptionFas_select"){


     if(this.getValue() == "ENABLED"){

      //Enable the disabled fields
      minNumberOfDigitsFas_numfield.enable();
      maxNumberOfDigitsFas_numfield.enable();
      endCollectionDigitsFas_text.enable();

      //specify default values if not defined
      if(minNumberOfDigitsFas_numfield.getValue() == '')
        minNumberOfDigitsFas_numfield.setValue(0);
      if(maxNumberOfDigitsFas_numfield.getValue() == '')
        maxNumberOfDigitsFas_numfield.setValue(0);
      if( endCollectionDigitsFas_text.getValue() == '')
        endCollectionDigitsFas_text.setValue("([0-9a-dA-D]+)?[\\#\\*]$");
    }else{
      //disable the enabled fields
      minNumberOfDigitsFas_numfield.disable();
      minNumberOfDigitsFas_numfield.clearInvalid();
      maxNumberOfDigitsFas_numfield.disable();
      maxNumberOfDigitsFas_numfield.clearInvalid();
      endCollectionDigitsFas_text.disable();
      endCollectionDigitsFas_text.clearInvalid();
    }

    
  }else if(this.id == "overlapDigitsReceptionNFas_select"){


     if(this.getValue() == "ENABLED"){

      //Enable the disabled fields
      minNumberOfDigitsNFas_numfield.enable();
      maxNumberOfDigitsNFas_numfield.enable();
      endCollectionDigitsNFas_text.enable();

      //specify default values if not defined
      if(minNumberOfDigitsNFas_numfield.getValue() == '')
        minNumberOfDigitsNFas_numfield.setValue(0);
      if(maxNumberOfDigitsNFas_numfield.getValue() == '')
        maxNumberOfDigitsNFas_numfield.setValue(0);
      if( endCollectionDigitsNFas_text.getValue() == '')
        endCollectionDigitsNFas_text.setValue("([0-9a-dA-D]+)?[\\#\\*]$");
    }else{
      //disable the enabled fields
      minNumberOfDigitsNFas_numfield.disable();
      minNumberOfDigitsNFas_numfield.clearInvalid();
      maxNumberOfDigitsNFas_numfield.disable();
      maxNumberOfDigitsNFas_numfield.clearInvalid();
      endCollectionDigitsNFas_text.disable();
      endCollectionDigitsNFas_text.clearInvalid();
    }



    
  }
}

//------------------------------------------------
// ISDN FAS config form
// -----------------------------------------------
function saveFasConfig(e){
  var owner = e.ownerCt;
  var form = owner.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "call/saveFasConfig";
  el.dom.method = 'POST';

  //var fasConfig = owner.getRawValues(false));
  var fasConfig = owner.getItemsValueAsJson();

  form.submit({
    params:{
      'fasConfig': fasConfig
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

//Common to Fas, Nfas, BRI, R2(CAS)
function applyToFas(e){
  var grid = e.ownerCt;
  var url;
  var params;
  var gridData;
  var records;
  var i;

  if( grid.getItemId() == "fasBChannelGrid") {
    url = Paraxip.baseUrl + "call/applyBChannelToFasConfig";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'fasConfigID':Ext.util.JSON.encode(fasID)};
  } else if( grid.id == "nfasBChannelGrid") {
    url = Paraxip.baseUrl + "call/applyBChannelToNfasConfig";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'nfasGroupID':Ext.util.JSON.encode(nfasID)};
  } else if( grid.id == "briBChannelGrid") {
    url = Paraxip.baseUrl + "call/applyBChannelToFasConfig";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'fasConfigID':Ext.util.JSON.encode(briID)};
  }else if( grid.id == "r2BChannelGrid") {
    url = Paraxip.baseUrl + "call/applyBChannelToR2Config";
    records = grid.getStore().getRange();
    gridData = new Array();
    for (i = 0; i < records.length; i++) {
      gridData.push(records[i].data);
    }
    gridData = Ext.util.JSON.encode(gridData);
    params = {'gridData':gridData,'r2ConfigID':Ext.util.JSON.encode(r2ID)};
  }

  Ext.Ajax.request({
    url: url,
    method: 'POST',
    disableCaching: false,
    params: params,
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      grid.ownerCt.hide();
      //reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

function addBchannel(e){
  var form = e.ownerCt;

  if( storeFasBoards.data.length < 1 ) {
    Ext.Msg.alert("No Digital Boards in config","There is no digital boards in your configuration."+
                  " Please add a board in the board section if you want to assign a board to this config.");
    return;
  }

  var firstBoardId = storeFasBoards.getAt(0).get('id');

  storeFasInterfaces.load({
    url: Paraxip.baseUrl + "call/getInterfaces",
    disableCaching: false,
    params: {'id':firstBoardId},
    callback: function(){
      if( form.id == 'fasBChannelGrid') {
        var newRecord = new Ext.data.Record({
          'boardId':firstBoardId,
          'interfaceId':storeFasInterfaces.getAt(0).get('id'),
          'firstChannel':1,
          'lastChannel':23,
          'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
        });
        fasBChannelGrid.getStore().add(newRecord);
        fasBChannelGrid.expand();
      } else if( form.id == 'nfasBChannelGrid' ) {
        var newRecord = new Ext.data.Record({
          'boardId':firstBoardId,
          'interfaceId':storeFasInterfaces.getAt(0).get('id'),
          'nfasInterfaceID':0,
          'DChannelMode':'DISABLED',
          'firstChannel':1,
          'lastChannel':23,
          'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
        });
        nfasBChannelGrid.getStore().add(newRecord);
        nfasBChannelGrid.expand();
      } else if( form.id == 'briBChannelGrid' ) {
        var newRecord = new Ext.data.Record({
          'boardId':firstBoardId,
          'interfaceId':storeFasInterfaces.getAt(0).get('id'),
          'firstChannel':1,
          'lastChannel':2,
          'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
        });
        briBChannelGrid.getStore().add(newRecord);
        briBChannelGrid.expand();
      } else if( form.id == 'r2BChannelGrid' ) {
        var newRecord = new Ext.data.Record({
          'boardId':firstBoardId,
          'interfaceId':storeFasInterfaces.getAt(0).get('id'),
          'firstChannel':1,
          'lastChannel':30,
          'channelGroupID':storeFasChannelGroup.getAt(0).get('id')
        });
        r2BChannelGrid.getStore().add(newRecord);
        r2BChannelGrid.expand();
      }
    }
  });

};

//Common to Fas, Nfas, BRI, R2
function deleteBchannels(e){
  var grid = e.ownerCt;

  var selectedRecords = grid.getSelectionModel().getSelections();

  if (selectedRecords !== []) {
    for( i = 0; i < selectedRecords.length; i++ ) {
      grid.getStore().remove(selectedRecords[i]);
    }
  }

};

var storeFasBoards = new Ext.data.JsonStore({
  id      : 'storeFasBoards',
  url     : Paraxip.baseUrl + "call/getBoards",
  root    : 'data',
  fields  : ['id','displayName','nodeId']
});

var storeFasInterfaces = new Ext.data.JsonStore({
  id      : 'storeFasInterfaces',
  url     : Paraxip.baseUrl + "call/getInterfaces",
  root    : 'data',
  fields  : ['id','displayName']
});

var storeFasChannelGroup = new Ext.data.JsonStore({
  id      : 'storeFasChannelGroup',
  url     : Paraxip.baseUrl + "call/getChannelGroups",
  root    : 'data',
  fields  : ['id']
});

var assignedBChannelFasWindow = new Ext.Window({
  title       : 'Assign BChannel to Fas Config',
  layout      : 'fit',
  layoutConfig: {
     autoWidth:false
  },
  width       : 600,
  height      : 500,
  closeAction : 'hide',
  resizable   : false,
  plain       : true,
  id          : "assignedBChannelFasWindow"
});

var fasBChannelGrid = new Ext.grid.EditorGridPanel({
  title       : 'BChannels assigned to config',
  id          : 'fasBChannelGrid',
  autoScroll  : true,
  width       : 550,
  height      : 400,
  enableColumnMove : false,
  selModel    : new Ext.grid.RowSelectionModel(),
  store       : new Ext.data.JsonStore({
    id:'storeFasBChannelGrid',
    url: Paraxip.baseUrl + "call/getFasBChannels",
    root: 'data',
    fields: [
      {name: 'boardId'},
      {name: 'interfaceId'},
      {name: 'firstChannel'},
      {name: 'lastChannel'},
      {name: 'channelGroupID'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     : [
    {
      header: "Board ID",
      width: 110,
      sortable: true,
      dataIndex: 'boardId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasBoards,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "Interface ID",
      width: 110,
      sortable: true,
      dataIndex: 'interfaceId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasInterfaces,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "First Channel",
      width: 110,
      sortable: true,
      dataIndex: 'firstChannel',
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Last Channel",
      width: 110,
      sortable: true,
      dataIndex: 'lastChannel',
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Channel Group ID",
      width: 110,
      sortable: true,
      dataIndex: 'channelGroupID',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasChannelGroup,
        listClass       : 'x-combo-list-small'
      })
    }
  ],
  listeners       : {
    'beforeedit':function(e){
      if( e.field == "interfaceId" ) {
        var colModel = e.grid.getColumnModel();
        var selectedRecord = e.grid.getSelectionModel().getSelected();
        var interfaceCellEditor = colModel.getCellEditor(e.column,e.row);
        interfaceCellEditor.field.store.load({
          url: Paraxip.baseUrl + "call/getInterfaces",
          disableCaching: false,
          params: {'id':selectedRecord.data.boardId}
        });
      }
    }
  },
  viewConfig:{forceFit:true}
});

//fasBChannelGrid.store.on('load',function(store){
//  store.each(function(record,index){
//    var colModel = fasBChannelGrid.getColumnModel();
//    colModel.getCellEditor(1,index).field.store.load({
//      url: Paraxip.baseUrl + "call/getInterfaces",
//      disableCaching: false,
//      params: {'id':record.data['boardId']}
//    });
//  });
//});

fasBChannelGrid.addButton("Add BChannel",addBchannel,fasBChannelGrid);
fasBChannelGrid.addButton("Delete",deleteBchannels,fasBChannelGrid);
fasBChannelGrid.addButton("Save",applyToFas,fasBChannelGrid);
fasBChannelGrid.addButton("Cancel",function(){assignedBChannelFasWindow.hide();});

assignedBChannelFasWindow.add(fasBChannelGrid);

var fasID = '';

var frmIsdnFas = new Ext.FormPanel({
  title         : "ISDN FAS config",
  frame         : true,
  header        : false,
  id            : "frmIsdnFas",
  autoScroll    : true,
  layout        : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },
  defaults      : {
    layout: 'form',
    style: 'text-align:left;',
    labelWidth: 150,
    bodyStyle: 'padding:5px;',
    width: 500,
    autoHeight: true,
    fill: true
  },
  buttonAlign     : "right",
  buttons         : [{
    id: "Paraxip.config.callControl.assignedFasButton",
    text: 'Assigned BChannel(s)',
    listeners: {
      'click':function(){
        fasID = idFas_text.getValue();
        fasBChannelGrid.getStore().load({
          url: Paraxip.baseUrl + "call/getFasBChannels",
          params: {'id':fasID}
        });
        storeFasBoards.load({
          url: Paraxip.baseUrl + "call/getBoards",
          params: {'id':fasID}
        });
        storeFasChannelGroup.load({
          url: Paraxip.baseUrl + "call/getChannelGroups",
          params: {'id':fasID}
        });
        assignedBChannelFasWindow.show();
      }
    }
  },{
    id: "Paraxip.config.callControl.saveFasButton",
    text: 'Save',
    listeners: {'click':saveFasConfig}
  },{
    id: "Paraxip.config.callControl.cancelFasButton",
    text: 'Cancel',
    listeners: {'click':callControlCancel}
  }]
});

var idFas_text = createIdText({
  id  : 'idFas_text'
});

var nameFas_text = createNameText({
  id  : 'nameFas_text'
});

var terminationFas_select = createIsdnTerminationSelection({
  id  : 'terminationFas_select'
});

var switchVariantFas_select = createIsdnSwitchVariantSelection({
  store: variantStoreFAS,
  id  : 'switchVariantFas_select'
});

var BChannelNegotiationFas_select =
                            createIsdnBChannelNegotiationSelection({
  id  : 'BChannelNegotiationBri_select',
  disabled  : true
});

var initRestartProcFas_select = createIsdnInitRestartProcSelection({
  id  : 'initRestartProcFas_select'
});

var inbandProgTonesGenFas_select = createIsdnInProgTonesGenSelection({
  id  : 'inbandProgTonesGenFas_select'
});

var inbandProgTonesIndFas_select = createIsdnInProgTonesIndSelection({
  id  : 'inbandProgTonesIndFas_select'
});

var callerNameLocationFas_select = createCallerNameLocationSelection({
  id        : 'callerNameLocationFas_select',
  listeners : {
    'select':isWaitFacilityDelayEnable,
    scope:callerNameLocationFas_select
  }
});

var waitFacilityDelayMsFas_numfield = createaWaitFacilityDelayMsNumberField({
  id  : 'waitFacilityDelayMsFas_numfield'
});


var FasPanel = new ParaxipPanelWithHelp({
  id    : 'FasPanel',
  name  : 'fas',
  title : 'Fas Configuration',
  helpText: "fasConfigPatternfasConfig",
  items : [{
    xtype: 'textfield',
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  }]
});


FasPanel.add(idFas_text);
FasPanel.add(nameFas_text);
FasPanel.add(terminationFas_select);
FasPanel.add(switchVariantFas_select);
FasPanel.add(BChannelNegotiationFas_select);
FasPanel.add(initRestartProcFas_select);
FasPanel.add(inbandProgTonesGenFas_select);
FasPanel.add(inbandProgTonesIndFas_select);
FasPanel.add(callerNameLocationFas_select);
FasPanel.add(waitFacilityDelayMsFas_numfield);
frmIsdnFas.add(FasPanel);


var FasOverlapDialingPanel = new ParaxipPanelWithHelp({
id    : 'FasOverlapDialingPanel',
name  : 'fasOverlap',
title : 'Overlap Dialing Configuration'
});


var overlapDigitsReceptionFas_select = createOverlapDigitsReceptionSelection({
  id  : 'overlapDigitsReceptionFas_select',
      listeners : { 
      'select':isOverlapDigitsReceptionEnabled,
       scope  :overlapDigitsReceptionFas_select
}
});

var minNumberOfDigitsFas_numfield = createOverlapDigitsReceptionMinDigitsField({
  id  : 'minNumberOfDigitsFas_numfield'
});

var maxNumberOfDigitsFas_numfield = createOverlapDigitsReceptionMaxDigitsField({
  id  : 'maxNumberOfDigitsFas_numfield'
});

var endCollectionDigitsFas_text = createOverlapDigitsReceptionEndCollectionDigitsText({
  id  : 'endCollectionDigitsFas_text'
});

FasOverlapDialingPanel.add(overlapDigitsReceptionFas_select);
FasOverlapDialingPanel.add(minNumberOfDigitsFas_numfield);
FasOverlapDialingPanel.add(maxNumberOfDigitsFas_numfield);
FasOverlapDialingPanel.add(endCollectionDigitsFas_text);
frmIsdnFas.add(FasOverlapDialingPanel);




/* JN TODO : applyToFasConfig  */


//------------------------------------------------
// ISDN NFAS config form
// -----------------------------------------------

var storeDchannelMode = ['DISABLED','PRIMARY'/*,'BACKUP'*/]; //BACKUP not supported for now

var assignedBChannelNfasWindow = new Ext.Window({
  title       : 'Assign BChannel to Nfas Group',
  layout      : 'fit',
  width       : 600,
  layoutConfig: {
    autoWidth:false
  },
  height      : 500,
  closeAction : 'hide',
  resizable   : false,
  plain       : true,
  id          : "assignedBChannelNfasWindow"
});

var nfasBChannelGrid = new Ext.grid.EditorGridPanel({
  title       : 'BChannels assigned to config',
  id          : 'nfasBChannelGrid',
  autoScroll  : true,
  width       : 500,
  height      : 400,
  enableColumnMove : false,
  selModel    : new Ext.grid.RowSelectionModel(),
  store       : new Ext.data.JsonStore({
    id:'storeNfasBChannelGrid',
    url: Paraxip.baseUrl + "call/getNfasBChannels",
    root: 'data',
    fields: [
      {name: 'boardId'},
      {name: 'interfaceId'},
      {name: 'nfasInterfaceID'},
      {name: 'DChannelMode'},
      {name: 'firstChannel'},
      {name: 'lastChannel'},
      {name: 'channelGroupID'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     :[
    {
      header: "Board ID",
      width: 95,
      sortable: true,
      dataIndex: 'boardId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasBoards,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "Interface ID",
      width: 95,
      sortable: true,
      dataIndex: 'interfaceId',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasInterfaces,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "NFasInterface ID",
      width: 115,
      sortable: true,
      dataIndex: 'nfasInterfaceID',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "DChannel Mode",
      width: 110,
      sortable: true,
      dataIndex: 'DChannelMode',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        store           : storeDchannelMode,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "First Channel",
      width: 100,
      sortable: true,
      dataIndex: 'firstChannel',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "Last Channel",
      width: 100,
      sortable: true,
      dataIndex: 'lastChannel',
      editor: new Ext.form.NumberField({
        allowNegative  : false,
        allowBlank  : false
      })
    },
    {
      header: "Channel Group ID",
      width: 105,
      sortable: true,
      dataIndex: 'channelGroupID',
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasChannelGroup,
        listClass       : 'x-combo-list-small'
      })
    }
  ],
  listeners       : {
    'beforeedit':function(e){
      if( e.field == "interfaceId" ) {
        var colModel = e.grid.getColumnModel();
        var selectedRecord = e.grid.getSelectionModel().getSelected();
        var interfaceCellEditor = colModel.getCellEditor(e.column,e.row);
        interfaceCellEditor.field.store.load({
          url: Paraxip.baseUrl + "call/getInterfaces",
          disableCaching: false,
          params: {'id':selectedRecord.data.boardId}
        });
      }
    }
  },
  viewConfig:{forceFit:true}
});

//nfasBChannelGrid.store.on('load',function(store){
//  store.each(function(record,index){
//    var colModel = nfasBChannelGrid.getColumnModel();
//    colModel.getCellEditor(1,index).field.store.load({
//      url: Paraxip.baseUrl + "call/getInterfaces",
//      disableCaching: false,
//      params: {'id':record.data['boardId']}
//    });
//  });
//});

nfasBChannelGrid.addButton("Add BChannel",addBchannel,nfasBChannelGrid);
nfasBChannelGrid.addButton("Delete",deleteBchannels,nfasBChannelGrid);
nfasBChannelGrid.addButton("Save",applyToFas,nfasBChannelGrid);
nfasBChannelGrid.addButton("Cancel",function(){assignedBChannelNfasWindow.hide();});

assignedBChannelNfasWindow.add(nfasBChannelGrid);

var nfasID = '';

function saveNfasConfig(e){
  var owner = e.ownerCt;
  var form = owner.getForm();
  var el = form.getEl();

  el.dom.action = Paraxip.baseUrl + "call/saveNfasConfig";
  el.dom.method = 'POST';

  var nfasGroup = owner.getItemsValueAsJson();

  form.submit({
    params:{
      'nfasGroup': nfasGroup
    },
    success: function(){
      Paraxip.isConfigOutOfSynch(true);
      reloadTree(true);
    },
    failure: Paraxip.failureMessage
  });

};

var frmIsdnNfas = new Ext.FormPanel({
  title         : "ISDN NFAS config",
  frame         : true,
  header        : false,
  id            : "frmIsdnNfas",
  autoScroll    : true,
  layout        : 'accordion',
  layoutConfig  : {
    activeItem  : 0
  },
  defaults      : {
    layout: 'form',
    style: 'text-align:left;',
    labelWidth: 150,
    bodyStyle: 'padding:5px;',
    width: 500,
    autoHeight: true,
    fill: true
  },
  buttonAlign   : "right",
  buttons       : [{
    id: "Paraxip.config.callControl.assignedNfasButton",
    text: 'Assigned BChannel(s)',
    listeners: {
      'click':function(){
        nfasID = idNfas_text.getValue();
        nfasBChannelGrid.getStore().load({
          url: Paraxip.baseUrl + "call/getNfasBChannels",
          params: {'id':nfasID}
        });
        storeFasBoards.load({
          url: Paraxip.baseUrl + "call/getBoards",
          params: {'id':nfasID}
        });
        storeFasChannelGroup.load({
          url: Paraxip.baseUrl + "call/getChannelGroups",
          params: {'id':nfasID}
        });
        assignedBChannelNfasWindow.show();
      }
    }
  },{
    id: "Paraxip.config.callControl.saveNfasButton",
    text: 'Save',
    listeners: {'click':saveNfasConfig}
  },{
    id: "Paraxip.config.callControl.cancelNfasButton",
    text: 'Cancel',
    listeners: {'click':callControlCancel}
  }]
});

var NfasPanel = new ParaxipPanelWithHelp({
  id    : 'NfasPanel',
  name  : 'nfas',
  title : 'Nfas Configuration',
  helpText: "nfasGroupnfasGroup",
  items : [{
    xtype: 'textfield',
    name: 'nodeId',
    hidden: true,
    hideLabel: true
  }]
});

var idNfas_text = createIdText({
  id  : 'idNfas_text'
});

var nameNfas_text = createNameText({
  id  : 'nameNfas_text'
});

var terminationNfas_select = createIsdnTerminationSelection({
  id  : 'terminationNfas_select'
});

var switchVariantNfas_select = createIsdnSwitchVariantSelection({
  store: variantStoreNFAS,
  id  : 'switchVariantNfas_select'
});

var BChannelNegotiationNfas_select =
                            createIsdnBChannelNegotiationSelection({
  id  : 'BChannelNegotiationBri_select',
  disabled  : true
});

var initRestartProcNfas_select = createIsdnInitRestartProcSelection({
  id  : 'initRestartProcNfas_select'
});

var inbandProgTonesGenNfas_select = createIsdnInProgTonesGenSelection({
  id  : 'inbandProgTonesGenNfas_select'
});

var inbandProgTonesIndNfas_select = createIsdnInProgTonesIndSelection({
  id  : 'inbandProgTonesIndNfas_select'
});

var callerNameLocationNfas_select = createCallerNameLocationSelection({
  id  : 'callerNameLocationNfas_select',
  listeners : {
    'select':isWaitFacilityDelayEnable,
    scope:callerNameLocationNfas_select
  }
});

var waitFacilityDelayMsNfas_numfield = createaWaitFacilityDelayMsNumberField({
  id  : 'waitFacilityDelayMsNfas_numfield'
});


NfasPanel.add(idNfas_text);
NfasPanel.add(nameNfas_text);
NfasPanel.add(terminationNfas_select);
NfasPanel.add(switchVariantNfas_select);
NfasPanel.add(BChannelNegotiationNfas_select);
NfasPanel.add(initRestartProcNfas_select);
NfasPanel.add(inbandProgTonesGenNfas_select);
NfasPanel.add(inbandProgTonesIndNfas_select);
NfasPanel.add(callerNameLocationNfas_select);
NfasPanel.add(waitFacilityDelayMsNfas_numfield);
frmIsdnNfas.add(NfasPanel);


var NFasOverlapDialingPanel = new ParaxipPanelWithHelp({
id    : 'NFasOverlapDialingPanel',
name  : 'nfasOverlap',
title : 'Overlap Dialing Configuration'
});


var overlapDigitsReceptionNFas_select = createOverlapDigitsReceptionSelection({
  id  : 'overlapDigitsReceptionNFas_select',
      listeners : { 
      'select':isOverlapDigitsReceptionEnabled,
       scope  :overlapDigitsReceptionNFas_select
}
});

var minNumberOfDigitsNFas_numfield = createOverlapDigitsReceptionMinDigitsField({
  id  : 'minNumberOfDigitsNFas_numfield'
});

var maxNumberOfDigitsNFas_numfield = createOverlapDigitsReceptionMaxDigitsField({
  id  : 'maxNumberOfDigitsNFas_numfield'
});

var endCollectionDigitsNFas_text = createOverlapDigitsReceptionEndCollectionDigitsText({
  id  : 'endCollectionDigitsNFas_text'
});

NFasOverlapDialingPanel.add(overlapDigitsReceptionNFas_select);
NFasOverlapDialingPanel.add(minNumberOfDigitsNFas_numfield);
NFasOverlapDialingPanel.add(maxNumberOfDigitsNFas_numfield);
NFasOverlapDialingPanel.add(endCollectionDigitsNFas_text);
frmIsdnNfas.add(NFasOverlapDialingPanel);


//------------------------------------------------
//ISDN BRI config form
//-----------------------------------------------
var assignedBChannelBriWindow = new Ext.Window({
  title       : 'Assign BChannel to BRI Config',
  layout      : 'fit',
  layoutConfig: {
     autoWidth:false
  },
  width       : 600,
  height      : 500,
  closeAction : 'hide',
  resizable   : false,
  plain       : true,
  id          : "assignedBChannelBriWindow"
});

var briBChannelGrid = new Ext.grid.EditorGridPanel({
  title       : 'BChannels assigned to config',
  id          : 'briBChannelGrid',
  autoScroll  : true,
  width       : 550,
  height      : 400,
  enableColumnMove : false,
  selModel    : new Ext.grid.RowSelectionModel(),
  store       : new Ext.data.JsonStore({
    id:'storeBriBChannelGrid',
    url: Paraxip.baseUrl + "call/getFasBChannels",
    root: 'data',
    fields: [
      {name: 'boardId'},
      {name: 'interfaceId'},
      {name: 'firstChannel'},
      {name: 'lastChannel'},
      {name: 'channelGroupID'},
      {name: 'nodeId'} //not displayed
    ]
  }),
  columns     : [
    {
      header: "Board ID",
      width: 110,
      sortable: true,
      dataIndex: 'boardId',
      renderer: renderEditableColumn,
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasBoards,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "Interface ID",
      width: 110,
      sortable: true,
      dataIndex: 'interfaceId',
      renderer: renderEditableColumn,
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasInterfaces,
        listClass       : 'x-combo-list-small'
      })
    },
    {
      header: "First Channel",
      width: 110,
      sortable: true,
      dataIndex: 'firstChannel',
      renderer: renderEditableColumn,
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Last Channel",
      width: 110,
      sortable: true,
      dataIndex: 'lastChannel',
      renderer: renderEditableColumn,
      editor: new Ext.form.NumberField({
        allowNegative : false,
        allowBlank  : false
      })
    },
    {
      header: "Channel Group ID",
      width: 110,
      sortable: true,
      dataIndex: 'channelGroupID',
      renderer: renderEditableColumn,
      editor: new Ext.form.ComboBox({
        forceSelection  : true,
        triggerAction   : 'all',
        mode            : 'local',
        selectOnFocus   : true,
        valueField      : 'id',
        displayField    : 'id',
        store           : storeFasChannelGroup,
        listClass       : 'x-combo-list-small'
      })
    }
  ],
  listeners       : {
    'beforeedit':function(e){
      if( e.field == "interfaceId" ) {
        var colModel = e.grid.getColumnModel();
        var selectedRecord = e.grid.getSelectionModel().getSelected();
        var interfaceCellEditor = colModel.getCellEditor(e.column,e.row);
        interfaceCellEditor.field.store.load({
          url: Paraxip.baseUrl + "call/getInterfaces",
          disableCaching: false,
          params: {'id':selectedRecord.data.boardId}
        });
      }
    }
  },
  viewConfig:{forceFit:true}
});

//fasBChannelGrid.store.on('load',function(store){
//  store.each(function(record,index){
//    var colModel = fasBChannelGrid.getColumnModel();
//    colModel.getCellEditor(1,index).field.store.load({
//      url: Paraxip.baseUrl + "call/getInterfaces",
//      disableCaching: false,
//      params: {'id':record.data['boardId']}
//    });
//  });
//});

briBChannelGrid.addButton("Add BChannel",addBchannel,briBChannelGrid);
briBChannelGrid.addButton("Delete",deleteBchannels,briBChannelGrid);
briBChannelGrid.addButton("Save",applyToFas,briBChannelGrid);
briBChannelGrid.addButton("Cancel",function(){assignedBChannelBriWindow.hide();});

assignedBChannelBriWindow.add(briBChannelGrid);

var briID = '';

function saveBriConfig(e){
var owner = e.ownerCt;
var form = owner.getForm();
var el = form.getEl();

el.dom.action = Paraxip.baseUrl + "call/saveBriConfig";
el.dom.method = 'POST';

var briConfig = owner.getItemsValueAsJson();

form.submit({
 params:{
   'briConfig': briConfig
 },
 success: function(){
   Paraxip.isConfigOutOfSynch(true);
   reloadTree(true);
 },
 failure: Paraxip.failureMessage
});

};

function briTopologyVisibleFields(){

  if( this.getValue() != "MULTIPOINT" ) {
    teiBri_numField.enable();
    // !+!jeleg: Fixed issue 6570 - BRI point to point: Value entered for TEI 
    // is always reset to 0 when clicking on save 
    //teiBri_numField.setValue(0);
  } else {
    teiBri_numField.disable();
    teiBri_numField.clearInvalid();
  }
}



var frmIsdnBri = new Ext.FormPanel({
title         : "ISDN BRI config",
frame         : true,
header        : false,
id            : "frmIsdnBri",
autoScroll    : true,
layout        : 'accordion',
layoutConfig  : {
 activeItem  : 0
},
defaults      : {
 layout: 'form',
 style: 'text-align:left;',
 labelWidth: 150,
 bodyStyle: 'padding:5px;',
 width: 500,
 autoHeight: true,
 fill: true
},
buttonAlign   : "right",
buttons       : [{
 id: "Paraxip.config.callControl.assignedBriButton",
 text: 'Assigned BChannel(s)',
 listeners: {
   'click':function(){
     briID= idBri_text.getValue();
     briBChannelGrid.getStore().load({
       url: Paraxip.baseUrl + "call/getFasBChannels",
       params: {'id':briID}
     });
     storeFasBoards.load({
       url: Paraxip.baseUrl + "call/getBoards",
       params: {'id':briID}
     });
     storeFasChannelGroup.load({
       url: Paraxip.baseUrl + "call/getChannelGroups",
       params: {'id':briID}
     });
     assignedBChannelBriWindow.show();
   }
 }
},{
 id: "Paraxip.config.callControl.saveBriButton",
 text: 'Save',
 listeners: {'click':saveBriConfig}
},{
 id: "Paraxip.config.callControl.cancelBriButton",
 text: 'Cancel',
 listeners: {'click':callControlCancel}
}]
});

var BriPanel = new ParaxipPanelWithHelp({
id    : 'BriPanel',
name  : 'bri',
title : 'BRI Configuration',
helpText: "briConfigPatternbriConfig",
items : [{
 xtype: 'textfield',
 name: 'nodeId',
 hidden: true,
 hideLabel: true
}]
});

var idBri_text = createIdText({
id  : 'idBri_text'
});

var nameBri_text = createNameText({
id  : 'nameBri_text'
});

var terminationBri_select = createIsdnTerminationSelection({
id  : 'terminationBri_select'
});

var switchVariantBri_select = createIsdnSwitchVariantSelection({
  store: variantStoreBRI,
  disabled:true,
  id  : 'switchVariantBri_select'
});

BriPanel.add(idBri_text);
BriPanel.add(nameBri_text);
BriPanel.add(terminationBri_select);
BriPanel.add(switchVariantBri_select);
frmIsdnBri.add(BriPanel);

var BriL2Panel = new ParaxipPanelWithHelp({
id    : 'BriL2Panel',
name  : 'briL2',
title : 'BRI L2 Configuration'
});

var topologyBri_select = createBriTopologySelection({
  disabled:false,
  id  : 'topologyBri_select',
  listeners: { 'select': briTopologyVisibleFields }
});

var teiBri_numField = createBriTeiNumberField({
id  : 'teiBri_numField'
});

BriL2Panel.add(topologyBri_select);
BriL2Panel.add(teiBri_numField);
frmIsdnBri.add(BriL2Panel);

var BriL3Panel = new ParaxipPanelWithHelp({
id    : 'BriL3Panel',
name  : 'briL3',
title : 'BRI L3 Configuration'
});


var BChannelNegotiationBri_select =
                         createIsdnBChannelNegotiationSelection({
id  : 'BChannelNegotiationBri_select',
disabled  : true
});

var initRestartProcBri_select = createIsdnInitRestartProcSelection({
id  : 'initRestartProcBri_select',
disabled  : true
});

var inbandProgTonesGenBri_select = createIsdnInProgTonesGenSelection({
id  : 'inbandProgTonesGenBri_select'
});

var inbandProgTonesIndBri_select = createIsdnInProgTonesIndSelection({
id  : 'inbandProgTonesIndBri_select'
});

var multipleSubscriberNumbersBri_text = createBriMultipleSubscriberNumbersText({
id  : 'multipleSubscriberNumbersBri_text'
});

var aocBehaviorBri_select = createAocBehaviorSelection({
  id  : 'aocBehaviorBri_select',
  disabled: true
});

var serviceTypeBri_select = createServiceTypeSelection({
  id  : 'serviceTypeBri_select'
});

BriL3Panel.add(BChannelNegotiationBri_select);
BriL3Panel.add(initRestartProcBri_select);
BriL3Panel.add(inbandProgTonesGenBri_select);
BriL3Panel.add(inbandProgTonesIndBri_select);
BriL3Panel.add(multipleSubscriberNumbersBri_text);
BriL3Panel.add(aocBehaviorBri_select);
BriL3Panel.add(serviceTypeBri_select);
frmIsdnBri.add(BriL3Panel);

var BriCallerNamePanel = new ParaxipPanelWithHelp({
id    : 'BriCallerNamePanel',
name  : 'briCallerName',
title : 'BRI ISDN Caller Name Configuration'
});

var callerNameLocationBri_select = createCallerNameLocationSelection({
id  : 'callerNameLocationBri_select',
listeners : {
 'select':isWaitFacilityDelayEnable,
 scope:callerNameLocationBri_select
}
});

var waitFacilityDelayMsBri_numfield = createaWaitFacilityDelayMsNumberField({
  id  : 'waitFacilityDelayMsBri_numfield'
});

BriCallerNamePanel.add(callerNameLocationBri_select);
BriCallerNamePanel.add(waitFacilityDelayMsBri_numfield);
frmIsdnBri.add(BriCallerNamePanel);



var BriOverlapDialingPanel = new ParaxipPanelWithHelp({
id    : 'BriOverlapDialingPanel',
name  : 'briOverlap',
title : 'Overlap Dialing Configuration'
});


var overlapDigitsReceptionBri_select = createOverlapDigitsReceptionSelection({
  id  : 'overlapDigitsReceptionBri_select',
      listeners : { 
      'select':isOverlapDigitsReceptionEnabled,
       scope  :overlapDigitsReceptionBri_select
}
});

var minNumberOfDigitsBri_numfield = createOverlapDigitsReceptionMinDigitsField({
  id  : 'minNumberOfDigitsBri_numfield'
});

var maxNumberOfDigitsBri_numfield = createOverlapDigitsReceptionMaxDigitsField({
  id  : 'maxNumberOfDigitsBri_numfield'
});

var endCollectionDigitsBri_text = createOverlapDigitsReceptionEndCollectionDigitsText({
  id  : 'endCollectionDigitsBri_text'
});

BriOverlapDialingPanel.add(overlapDigitsReceptionBri_select);
BriOverlapDialingPanel.add(minNumberOfDigitsBri_numfield);
BriOverlapDialingPanel.add(maxNumberOfDigitsBri_numfield);
BriOverlapDialingPanel.add(endCollectionDigitsBri_text);
frmIsdnBri.add(BriOverlapDialingPanel);
