//-----------------------------------------------
// Function used to disable/enable every button
// from a radiogroup.
//-----------------------------------------------
var disablingLoop = function(e, isVisible) {

  e.setVisible(isVisible);
  e.setDisabled(!isVisible);
};
//-----------------------------------------------
// Main form that contains all the fields
//-----------------------------------------------
Ext.namespace("Paraxip.wizard");

// In Internet Explorer, the css rendering for this form is slightly different
// than on Firefox and Chrome so we need to add some padding at the top to make
// the UI look the same.
if( Ext.isIE ) {
  var bodyStyleStr = "padding-top:13px;";
} else {
  var bodyStyleStr = "";
}

var isdn_form = new Ext.Panel({
  title           : "PSTN Digital Configuration"
  ,id             : "Paraxip.wizard.isdn_form"
  ,frame          : true
  ,standardSubmit : true
  ,header         : false
  ,bodyStyle      : "padding-left:14px;" + bodyStyleStr
  ,layout         : 'form'
  
  ,defaults       : {
    autoHeight: true
    ,autoWidth: true
  }
});

var digitalDescriptionLabel = new Ext.form.Label({
  id      : "Paraxip.wizard.DigitDescriptionLabel"
  ,text   : "Select the PSTN digital configuration"
  ,style  : "font-size:120%;font-weight:bold;line-height:60px;"
});


//-----------------------------------------------
// Interface Type fields
//-----------------------------------------------
Paraxip.wizard.radio_t1 = new Ext.form.Radio({
  boxLabel    : 'T1',
  id          : 'Radio_T1',
  name        : 'interface_type',
  inputValue  : 'T1',
  listeners   : {'check':interfaceCheck},
  checked     : 'true'
});
var radio_e1 = new Ext.form.Radio({
  boxLabel    : 'E1',
  id          : 'Radio_E1',
  name        : 'interface_type',
  inputValue  : 'E1',
  width       : 235,
  listeners   : {'check':interfaceCheck}
});
var radio_j1 = new Ext.form.Radio({
  boxLabel    : 'J1',
  id          : 'Radio_J1',
  name        : 'interface_type',
  inputValue  : 'J1',
  listeners   : {'check':interfaceCheck}
});

function interfaceCheck() {
  var isT1Visible = Paraxip.wizard.radio_t1.getValue();
  var isJ1Visible = radio_j1.getValue();
  var isE1Visible = radio_e1.getValue();
  
  //Signaling Check
  var isE1_ISDN_Variant_Visible = Paraxip.wizard.e1CCS.getValue();
  var isE1_CAS_Variant_Visible = e1CAS.getValue();

  non_crc4.setValue(isE1_CAS_Variant_Visible);
  Paraxip.wizard.crc4.setValue(isE1_ISDN_Variant_Visible);
  
  //T1 interface's fields
  disablingLoop(framing, isT1Visible);
  disablingLoop(encoding, isT1Visible);
  disablingLoop(isdn_sw, isT1Visible);
  
  //E1 interface's fields
  disablingLoop(framingE1, isE1Visible);
  disablingLoop(encodingE1, isE1Visible);
  disablingLoop(isdnSwE1, isE1Visible && isE1_ISDN_Variant_Visible);
  disablingLoop(casSwE1, isE1Visible && isE1_CAS_Variant_Visible);
  disablingLoop(signalingE1, isE1Visible);
  
  //J1 interface's fields
  disablingLoop(framingJ1, isJ1Visible);
  disablingLoop(encodingJ1, isJ1Visible);
  disablingLoop(isdnSwJ1, isJ1Visible);
  
  
}
var interfaces = new Ext.form.FieldSet({
  id          : "interfaces",
  title       : "Interface Type",
  layout      : "column"
});

interfaces.add(Paraxip.wizard.radio_t1);
interfaces.add(radio_e1);
interfaces.add(radio_j1);

//-----------------------------------------------
// Framing Type fields
//-----------------------------------------------
Paraxip.wizard.esf = new Ext.form.Radio({id:'esf_radio', boxLabel: 'ESF', name: 'framing_T1',inputValue: 'ESF', checked: 'true'});

var d4 = new Ext.form.Radio({id:'d4_radio', boxLabel: 'D4', name: 'framing_T1',inputValue: 'D4', width: 225});

//ESF_Japan not supported for now
//var esf_japan = new Ext.form.Radio({id:'esf_japan_radio', boxLabel: 'ESF Japan', name: 'framing_T1',inputValue: 'ESF_Japan', width: 150});

Paraxip.wizard.crc4 = new Ext.form.Radio({id:'crc4_radio', boxLabel: 'CRC4', name: 'framing_E1',inputValue: 'CRC4', checked: 'true'});

var non_crc4 = new Ext.form.Radio({id:'non_crc4_radio', boxLabel: 'NON CRC4', name: 'framing_E1',inputValue: 'NonCRC4', width: 245});

Paraxip.wizard.j1_esf = new Ext.form.Radio({id:'j1_esf_radio', boxLabel: 'ESF', name: 'framing_J1',inputValue: 'ESF', checked: 'true'});
var j1_d4 = new Ext.form.Radio({id:'j1_d4_radio', boxLabel: 'D4', name: 'framing_J1',inputValue: 'D4', width: 225});

var framing = new Ext.form.FieldSet({
  title       : "Framing Type",
  id          : "framingT1",
  layout      : "column"
});
var framingJ1 = new Ext.form.FieldSet({
  title       : "Framing Type",
  id          : "framingJ1",
  hidden      : true,
  layout      : "column"
});
var framingE1 = new Ext.form.FieldSet({
  title       : "Framing Type",
  id          : "framingE1",
  hidden      : true,
  layout      : "column"
});

framing.add(Paraxip.wizard.esf);
framing.add(d4);
framingJ1.add(Paraxip.wizard.j1_esf);
framingJ1.add(j1_d4);
framingE1.add(Paraxip.wizard.crc4);
framingE1.add(non_crc4);

//Uncomment when esf_japan will be supported
//framing.add(esf_japan);

//-----------------------------------------------
// Encoding Type fields
//-----------------------------------------------
Paraxip.wizard.b8zs = new Ext.form.Radio({id:'b8zs_radio', boxLabel: 'B8ZS', name: 'encoding_T1',inputValue: 'B8ZS', checked: 'true'});
var ami_t1 = new Ext.form.Radio({id:'ami_t1_radio', boxLabel: 'AMI', name: 'encoding_T1',inputValue: 'AMI', width: 220});

Paraxip.wizard.hdb3 = new Ext.form.Radio({id:'hdb3_radio', boxLabel: 'HDB3', name: 'encoding_E1',inputValue: 'HDB3', checked: 'true'});
var ami_e1 = new Ext.form.Radio({id:'ami_e1_radio', boxLabel: 'AMI', name: 'encoding_E1',inputValue: 'AMI', width: 217});

Paraxip.wizard.j1_b8zs = new Ext.form.Radio({id:'j1_b8zs_radio', boxLabel: 'B8ZS', name: 'encoding_J1',inputValue: 'B8ZS', checked: 'true'});
var j1_ami = new Ext.form.Radio({id:'j1_ami_radio', boxLabel: 'AMI', name: 'encoding_J1',inputValue: 'AMI', width: 220});


var encoding = new Ext.form.FieldSet({
  title       : "Encoding Type",
  id          : "encodingT1",
  layout      : "column"
});
var encodingJ1 = new Ext.form.FieldSet({
  title       : "Encoding Type",
  id          : "encodingJ1",
  hidden      : true,
  layout      : "column"
});
var encodingE1 = new Ext.form.FieldSet({
  title       : "Encoding Type",
  id          : "encodingE1",
  hidden      : true,
  layout      : "column"
});


encoding.add(Paraxip.wizard.b8zs);
encoding.add(ami_t1);
encodingJ1.add(Paraxip.wizard.j1_b8zs);
encodingJ1.add(j1_ami);
encodingE1.add(Paraxip.wizard.hdb3);
encodingE1.add(ami_e1);



//-----------------------------------------------
//Signaling Type fields
//-----------------------------------------------
Paraxip.wizard.e1CCS = new Ext.form.Radio({
  id:'e1_ccs_radio', 
  boxLabel: 'CCS (PRI ISDN)', 
  name: 'signaling_E1',
  inputValue: 'CCS', 
  checked: 'true',
  listeners   : {'check':interfaceCheck}
});

var e1CAS = new Ext.form.Radio({
  id:'e1_cas_radio', 
  boxLabel: 'CAS (MFC/R2)', 
  name: 'signaling_E1',
  inputValue: 'CAS', 
  width: 170,
  listeners   : {'check':interfaceCheck}
});

var signalingE1 = new Ext.form.FieldSet({
  title       : "Signaling Type",
  id          : "signalingE1",
  hidden      : true,
  layout      : "column"
});

signalingE1.add(Paraxip.wizard.e1CCS);
signalingE1.add(e1CAS);

Ext.Ajax.request({
  url: Paraxip.baseUrl + "getGCParameterValue",
  disableCaching: true,
  params: {"paramName":'netborder.oamWebServer.hideR2Options'},
  success: function(result){
    var response = Ext.util.JSON.decode(result.responseText);
    if( response.paramValue ) {
      signalingE1.remove(e1CAS);
    }
  }
});
//-----------------------------------------------
// ISDN Switch variant fields
//-----------------------------------------------
Paraxip.wizard.ess = new Ext.form.Radio({id:'4ess_radio', boxLabel: '4ESS', name: 'isdn_sw_T1',inputValue: '4ESS'});

var lucent = new Ext.form.Radio({id:'lucent_radio', boxLabel: 'Lucent 5ESS', name: 'isdn_sw_T1',inputValue: '5ESS', checked: 'true', width: 260});

Paraxip.wizard.etsi = new Ext.form.Radio({id:'etsi_radio', boxLabel: 'NET5 (ETSI, EuroISDN)', name: 'isdn_sw_E1',inputValue: 'ETSI', checked: 'true'});

var nortel = new Ext.form.Radio({id:'nortel_radio', boxLabel: 'Nortel DMS100', name: 'isdn_sw_T1', inputValue: 'DMS100'});

var ni2 = new Ext.form.Radio({id:'ni2_radio', boxLabel: 'National ISDN 2 (NI2)', name: 'isdn_sw_T1', inputValue: 'NI2', width: 300});

Paraxip.wizard.INS1500 = new Ext.form.Radio({id:'j1_INS1500_radio', boxLabel: 'INS1500 (NTT-Japan)', name: 'isdn_sw_J1',inputValue: 'INS1500', checked: 'true'});

Paraxip.wizard.countryR2 = new Ext.form.ComboBox({
  id              : 'countryR2'
  ,fieldLabel     : "Country"
  ,store           : ['Argentina',
                     'Brazil',
                     'Czech Republic',
                     'Colombia',
                     'Ecuador',
                     'Mexico',
                     'Philippines',
                     'Venezuela',
                     'ITU Standard']
  ,valueField     : 'countryR2'
  ,displayField   : 'countryR2'
  ,forceSelection : true
  ,triggerAction  : 'all'
  ,mode           : 'local'
  ,typeAhead      : true
  ,selectOnFocus  : true
  ,allowBlank     : false
  ,width          : 145
  ,minListWidth   : 145
  ,forceSelection : true
  //,helpUrl        : "casCallControlConfigcasdigitalR2ConfigswitchVariant"
  ,style          : ".x-form-text {margin: 0px 0;}"
  ,listeners      : {'render': function(){
      this.setValue("ITU Standard");
   }
  }
});


var isdn_sw = new Ext.form.FieldSet({
  title       : "ISDN Switch Variant",
  id          : "isdnSwT1",
  layout      : "column"
});
var isdnSwJ1 = new Ext.form.FieldSet({
  title       : "ISDN Switch Variant",
  id          : "isdnSwJ1",
  hidden      : true,
  layout      : "column"
});
var isdnSwE1 = new Ext.form.FieldSet({
  title       : "ISDN Switch Variant",
  id          : "isdnSwE1",
  hidden      : true,
  layout      : "column"
});
var casSwE1 = new Ext.form.FieldSet({
  id          : "casSwE1"
  ,title      : "CAS (MFC/R2) Variant"
  ,autoHeight : true
  ,items      : [Paraxip.wizard.countryR2]
  ,hidden     : true
  ,layout     : 'form'
  ,style      : 'text-align:left;'
  ,labelWidth : 100
  ,bodyStyle  : 'padding:5px;'
});

isdn_sw.add(Paraxip.wizard.ess);
isdn_sw.add(lucent);
isdn_sw.add(nortel);
isdn_sw.add(ni2);
isdnSwE1.add(Paraxip.wizard.etsi);
//casSwE1.add(switchVariantR2_select);
isdnSwJ1.add(Paraxip.wizard.INS1500);
isdn_form.add(digitalDescriptionLabel);
isdn_form.add(interfaces);
isdn_form.add(signalingE1);
isdn_form.add(framing);
isdn_form.add(framingJ1);
isdn_form.add(framingE1);
isdn_form.add(encoding);
isdn_form.add(encodingJ1);
isdn_form.add(encodingE1);
isdn_form.add(isdn_sw);
isdn_form.add(isdnSwJ1);
isdn_form.add(isdnSwE1);
isdn_form.add(casSwE1);





